@@ -32,7 +32,7 @@ pub use capi::pa_prop_type_t as PropType;
 /// Represents the type of encoding used in a stream or accepted by a sink.
 #[repr(C)]
 #[non_exhaustive]
-#[derive(Debug, Copy, Clone, PartialEq, Eq)]
+#[derive(Debug, Default, Copy, Clone, PartialEq, Eq)]
 #[derive(FromPrimitive, ToPrimitive)]
 #[allow(non_camel_case_types)]
 pub enum Encoding {
@@ -62,6 +62,7 @@ pub enum Encoding {
     DTSHD_IEC61937,
 
     /// Represents an invalid encoding.
+    #[default]
     Invalid = -1,
 }
 
@@ -100,18 +101,11 @@ impl From<capi::pa_encoding_t> for Encoding {
     }
 }
 
-impl Default for Encoding {
-    #[inline(always)]
-    fn default() -> Self {
-        Encoding::Invalid
-    }
-}
-
 /// Represents the format of data provided in a stream or processed by a sink.
 pub struct Info {
     /// The actual C object.
     pub(crate) ptr: *mut InfoInternal,
-    /// Wrapped property list pointer.
+    /// A nicely wrapped copy of it’s property list pointer, for convenience.
     properties: Proplist,
     /// Used to avoid freeing the internal object when used as a weak wrapper in callbacks.
     weak: bool,
@@ -217,9 +211,10 @@ impl Info {
     /// Creates a new `Info` from an existing [`InfoInternal`] pointer.
     pub(crate) fn from_raw(ptr: *mut InfoInternal) -> Self {
         assert_eq!(false, ptr.is_null());
-        // Note, yes, this should be using `from_raw_weak()`, the ‘free’ function for a format info
-        // object free’s its own proplist!
-        unsafe { Self { ptr: ptr, properties: Proplist::from_raw_weak((*ptr).list), weak: false } }
+        // Note, yes, this should be weak; the ‘free’ function for a format info object free’s its
+        // own proplist!
+        let pl = Proplist::from_raw_weak(unsafe { (*ptr).list });
+        Self { ptr: ptr, properties: pl, weak: false }
     }
 
     /// Creates a new `Info` from an existing [`InfoInternal`] pointer.
@@ -227,7 +222,17 @@ impl Info {
     /// This is the ‘weak’ version, which avoids destroying the internal object when dropped.
     pub(crate) fn from_raw_weak(ptr: *mut InfoInternal) -> Self {
         assert_eq!(false, ptr.is_null());
-        unsafe { Self { ptr: ptr, properties: Proplist::from_raw_weak((*ptr).list), weak: true } }
+        let pl = Proplist::from_raw_weak(unsafe { (*ptr).list });
+        Self { ptr: ptr, properties: pl, weak: true }
+    }
+
+    /// Returns a new `Info` struct representing the same format.
+    ///
+    /// If this is called on a ‘weak’ instance, a non-weak object is returned.
+    #[inline]
+    pub(crate) fn to_owned(&self) -> Self {
+        let ptr = unsafe { capi::pa_format_info_copy(self.ptr as *const capi::pa_format_info) };
+        Self::from_raw(ptr as *mut InfoInternal)
     }
 
     /// Checks whether the `Info` structure is valid.
@@ -492,7 +497,10 @@ impl Info {
     }
 
     /// Sets a property with a list of integer values.
+    ///
+    /// Panics if length of array is larger than `i32::MAX`.
     pub fn set_prop_int_array(&mut self, key: &str, values: &[i32]) {
+        assert!(values.len() <= i32::MAX as usize);
         // Warning: New CStrings will be immediately freed if not bound to a variable, leading to
         // as_ptr() giving dangling pointers!
         let c_key = CString::new(key).unwrap();
@@ -520,7 +528,10 @@ impl Info {
     }
 
     /// Sets a property with a list of string values.
+    ///
+    /// Panics if length of array is larger than `i32::MAX`.
     pub fn set_prop_string_array(&mut self, key: &str, values: &[&str]) {
+        assert!(values.len() <= i32::MAX as usize);
         // Warning: New CStrings will be immediately freed if not bound to a variable, leading to
         // as_ptr() giving dangling pointers!
         let c_key = CString::new(key).unwrap();
@@ -566,9 +577,11 @@ impl Info {
     /// Note for PCM: If the channel count is left unspecified in the `Info` object, then the server
     /// will select the stream channel count. In that case the stream channel count will most likely
     /// match the device channel count, meaning that up/downmixing will be avoided.
+    ///
+    /// Panics if `channels` is larger than can be submitted to the C function.
     #[inline]
     pub fn set_channels(&mut self, channels: u32) {
-        debug_assert!(channels <= std::i32::MAX as u32);
+        assert!(channels <= std::i32::MAX as u32);
         unsafe { capi::pa_format_info_set_channels(self.ptr as *mut capi::pa_format_info,
             channels as i32) }
     }
@@ -594,11 +607,10 @@ impl Drop for Info {
 }
 
 impl Clone for Info {
-    /// Returns a new `Info` struct and representing the same format. If this is called on a ‘weak’
-    /// instance, a non-weak object is returned.
+    /// Returns a new `Info` struct and representing the same format.
+    ///
+    /// If this is called on a ‘weak’ instance, a non-weak object is returned.
     fn clone(&self) -> Self {
-        let ptr = unsafe { capi::pa_format_info_copy(self.ptr as *const capi::pa_format_info) };
-        assert_eq!(false, ptr.is_null());
-        Self::from_raw(ptr as *mut InfoInternal)
+        self.to_owned()
     }
 }
