@@ -57,12 +57,12 @@ impl FromCStr for Option<String> {
 }
 
 trait ToCStr {
-    fn to_c_str(&self) -> CString;
+    fn to_c_str(&self) -> Result<CString, std::ffi::NulError>;
 }
 
 impl <'a>ToCStr for &'a str {
-    fn to_c_str(&self) -> CString {
-        CString::new(*self).unwrap()
+    fn to_c_str(&self) -> Result<CString, std::ffi::NulError> {
+        CString::new(*self)
     }
 }
 
@@ -91,12 +91,12 @@ pub fn addchstr(s: &[chtype]) -> i32
 { unsafe { ll::addchstr(s.as_ptr()) } }
 
 
-pub fn addnstr(s: &str, n: i32) -> i32
-{ unsafe { ll::addnstr(s.to_c_str().as_ptr(), n) } }
+pub fn addnstr(s: &str, n: i32) -> Result<i32, std::ffi::NulError>
+{ unsafe { Ok(ll::addnstr(s.to_c_str()?.as_ptr(), n) )} }
 
 
-pub fn addstr(s: &str) -> i32
-{ unsafe { ll::addstr(s.to_c_str().as_ptr()) } }
+pub fn addstr(s: &str) -> Result<i32, std::ffi::NulError>
+{ unsafe { Ok(ll::addstr(s.to_c_str()?.as_ptr())) } }
 
 
 pub fn assume_default_colors(fg: i32, bg: i32) -> i32
@@ -288,8 +288,8 @@ pub fn endwin() -> i32
 { unsafe { ll::endwin() } }
 
 
-pub fn erasechar() -> char
-{ unsafe { char::from_u32(ll::erasechar() as u32).expect("Invalid char") } }
+pub fn erasechar() -> Option<char>
+{ unsafe { char::from_u32(ll::erasechar() as u32) } }
 
 
 pub fn filter()
@@ -400,14 +400,22 @@ pub fn getnstr(s: &mut String, n: i32) -> i32
 
 pub fn getstr(s: &mut String) -> i32
 {
-  /* XXX: This is probably broken. */
   let mut ch = getch();
+  let mut v = Vec::new();
   while ch != '\n' as i32 && ch != '\r' as i32
   {
-    unsafe { s.as_mut_vec().push(ch as u8); }
+    v.push(ch as u8);
     ch = getch();
   }
-  OK
+  match std::str::from_utf8(&v) {
+    Ok(parsed) => {
+      *s = parsed.to_string();
+      OK
+    },
+    Err(_) => {
+      ERR
+    },
+  }
 }
 
 
@@ -593,26 +601,6 @@ pub fn insstr(s: &str) -> i32
 }
 
 
-pub fn instr(s: &mut String) -> i32
-{
-  /* XXX: This is probably broken. */
-  unsafe
-  {
-    let buf = s.as_bytes().as_ptr();
-    let ret = ll::instr(mem::transmute(buf));
-
-    let capacity = s.capacity();
-    match s.find('\0')
-    {
-      Some(index) => s.as_mut_vec().set_len(index as usize),
-      None => s.as_mut_vec().set_len(capacity),
-    }
-
-    ret
-  }
-}
-
-
 pub fn intrflush(w: WINDOW, bf: bool) -> i32
 { unsafe { ll::intrflush(w, bf as ll::c_bool) } }
 
@@ -681,8 +669,8 @@ pub fn keypad(w: WINDOW, bf: bool) -> i32
 { unsafe { ll::keypad(w, bf as ll::c_bool) } }
 
 
-pub fn killchar() -> char
-{ unsafe { char::from_u32(ll::killchar() as u32).expect("Invalid char") } }
+pub fn killchar() -> Option<char>
+{ unsafe { char::from_u32(ll::killchar() as u32) } }
 
 
 pub fn leaveok(w: WINDOW, bf: bool) -> i32
@@ -721,18 +709,18 @@ pub fn mvaddchstr(y: i32, x: i32, s: &[chtype]) -> i32
 }
 
 
-pub fn mvaddnstr(y: i32, x: i32, s: &str, n: i32) -> i32
+pub fn mvaddnstr(y: i32, x: i32, s: &str, n: i32) -> Result<i32, std::ffi::NulError>
 {
   if mv(y, x) == ERR
-  { return ERR; }
+  { return Ok(ERR); }
   addnstr(s, n)
 }
 
 
-pub fn mvaddstr(y: i32, x: i32, s: &str) -> i32
+pub fn mvaddstr(y: i32, x: i32, s: &str) -> Result<i32, std::ffi::NulError>
 {
   if mv(y, x) == ERR
-  { return ERR; }
+  { return Ok(ERR); }
   addstr(s)
 }
 
@@ -827,18 +815,10 @@ pub fn mvinsstr(y: i32, x: i32, s: &str) -> i32
 }
 
 
-pub fn mvinstr(y: i32, x: i32, s: &mut String) -> i32
+pub fn mvprintw(y: i32, x: i32, s: &str) -> Result<i32, std::ffi::NulError>
 {
   if mv(y, x) == ERR
-  { return ERR; }
-  instr(s)
-}
-
-
-pub fn mvprintw(y: i32, x: i32, s: &str) -> i32
-{
-  if mv(y, x) == ERR
-  { return ERR; }
+  { return Ok(ERR); }
   printw(s)
 }
 
@@ -859,12 +839,12 @@ pub fn mvwaddchstr(w: WINDOW, y: i32, x: i32, s: &[chtype]) -> i32
 { unsafe { ll::mvwaddchstr(w, y, x, s.as_ptr()) } }
 
 
-pub fn mvwaddnstr(w: WINDOW, y: i32, x: i32, s: &str, n: i32) -> i32
-{ unsafe { ll::mvwaddnstr(w, y, x, s.to_c_str().as_ptr(), n) } }
+pub fn mvwaddnstr(w: WINDOW, y: i32, x: i32, s: &str, n: i32) -> Result<i32, std::ffi::NulError>
+{ unsafe { Ok(ll::mvwaddnstr(w, y, x, s.to_c_str()?.as_ptr(), n)) } }
 
 
-pub fn mvwaddstr(w: WINDOW, y: i32, x: i32, s: &str) -> i32
-{ unsafe { ll::mvwaddstr(w, y, x, s.to_c_str().as_ptr()) } }
+pub fn mvwaddstr(w: WINDOW, y: i32, x: i32, s: &str) -> Result<i32, std::ffi::NulError>
+{ unsafe { Ok(ll::mvwaddstr(w, y, x, s.to_c_str()?.as_ptr())) } }
 
 
 pub fn mvwchgat(w: WINDOW, y: i32, x: i32, n: i32, attr: attr_t, color: i16) -> i32
@@ -891,17 +871,25 @@ pub fn mvwgetnstr(w: WINDOW, y: i32, x: i32, s: &mut String, n: i32) -> i32
 
 pub fn mvwgetstr(w: WINDOW, y: i32, x: i32, s: &mut String) -> i32
 {
-  if mv(y, x) == ERR
-  { return ERR; }
+    if mv(y, x) == ERR
+    { return ERR; }
 
-  /* XXX: This is probably broken. */
-  let mut ch = wgetch(w);
-  while ch != '\n' as i32 && ch != '\r' as i32
-  {
-    unsafe { s.as_mut_vec().push(ch as u8); }
-    ch = wgetch(w);
-  }
-  OK
+    let mut ch = wgetch(w);
+    let mut v = Vec::new();
+    while ch != '\n' as i32 && ch != '\r' as i32
+    {
+	v.push(ch as u8);
+	ch = wgetch(w);
+    }
+    match std::str::from_utf8(&v) {
+	Ok(parsed) => {
+	    *s = parsed.to_string();
+	    OK
+	},
+	Err(_) => {
+	    ERR
+	},
+    }
 }
 
 
@@ -983,12 +971,12 @@ pub fn mvwinsch(w: WINDOW, y: i32, x: i32, ch: chtype) -> i32
 { unsafe { ll::mvwinsch(w, y, x, ch) } }
 
 
-pub fn mvwinsnstr(w: WINDOW, y: i32, x: i32, s: &str, n: i32) -> i32
-{ unsafe { ll::mvwinsnstr(w, y, x, s.to_c_str().as_ptr(), n) } }
+pub fn mvwinsnstr(w: WINDOW, y: i32, x: i32, s: &str, n: i32) -> Result<i32, std::ffi::NulError>
+{ unsafe { Ok(ll::mvwinsnstr(w, y, x, s.to_c_str()?.as_ptr(), n)) } }
 
 
-pub fn mvwinsstr(w: WINDOW, y: i32, x: i32, s: &str) -> i32
-{ unsafe { ll::mvwinsstr(w, y, x, s.to_c_str().as_ptr()) } }
+pub fn mvwinsstr(w: WINDOW, y: i32, x: i32, s: &str) -> Result<i32, std::ffi::NulError>
+{ unsafe { Ok(ll::mvwinsstr(w, y, x, s.to_c_str()?.as_ptr())) } }
 
 
 pub fn mvwinstr(w: WINDOW, y: i32, x: i32, s: &mut String) -> i32
@@ -1011,8 +999,13 @@ pub fn mvwinstr(w: WINDOW, y: i32, x: i32, s: &mut String) -> i32
 }
 
 
-pub fn mvwprintw(w: WINDOW, y: i32, x: i32, s: &str) -> i32
-{ unsafe { ll::mvwprintw(w, y, x, s.to_c_str().as_ptr()) } }
+pub fn mvwprintw(w: WINDOW, y: i32, x: i32, s: &str) -> Result<i32, std::ffi::NulError> {
+    // We don't actually need this function to support format strings,
+    // as they are more safely done in Rust.
+    unsafe {
+	Ok(ll::mvwprintw(w, y, x, "%s".to_c_str()?.as_ptr(), s.to_c_str()?.as_ptr()))
+    }
+}
 
 
 pub fn mvwvline(w: WINDOW, y: i32, x: i32, ch: chtype, n: i32) -> i32
@@ -1027,14 +1020,14 @@ pub fn newpad(lines: i32, cols: i32) -> WINDOW
 { unsafe { ll::newpad(lines, cols) } }
 
 
-pub fn newterm(ty: Option<&str>, out_fd: FILE_p, in_fd: FILE_p) -> SCREEN
+pub fn newterm(ty: Option<&str>, out_fd: FILE_p, in_fd: FILE_p) -> Result<SCREEN, std::ffi::NulError>
 {
   unsafe
   {
-    match ty {
-      Some(s) => ll::newterm(s.to_c_str().as_ptr(), out_fd, in_fd),
+    Ok(match ty {
+      Some(s) => ll::newterm(s.to_c_str()?.as_ptr(), out_fd, in_fd),
       None    => ll::newterm(std::ptr::null(), out_fd, in_fd),
-    }
+    })
   }
 }
 
@@ -1103,13 +1096,19 @@ pub fn prefresh(pad: WINDOW, pmin_row: i32, pmin_col: i32, smin_row: i32, smin_c
 { unsafe { ll::prefresh(pad, pmin_row, pmin_col, smin_row, smin_col, smax_row, smax_col) } }
 
 
-#[deprecated(since = "5.98.0", note = "printw can segfault when printing string that contains % sign. Use addstr instead")]
-pub fn printw(s: &str) -> i32
-{ unsafe { ll::printw(s.to_c_str().as_ptr()) } }
+#[deprecated(since = "5.98.0", note = "printw format support is disabled. Use addstr instead")]
+pub fn printw(s: &str) -> Result<i32, std::ffi::NulError>
+{
+    // We don't actually need this function to support format strings,
+    // as they are more safely done in Rust.
+    unsafe {
+	Ok(ll::printw("%s".to_c_str()?.as_ptr(), s.to_c_str()?.as_ptr()))
+    }
+}
 
 
-pub fn putp(s: &str) -> i32
-{ unsafe { ll::putp(s.to_c_str().as_ptr()) } }
+pub fn putp(s: &str) -> Result<i32, std::ffi::NulError>
+{ unsafe { Ok(ll::putp(s.to_c_str()?.as_ptr())) } }
 
 
 pub fn putwin(w: WINDOW, f: FILE_p) -> i32
@@ -1156,12 +1155,12 @@ pub fn savetty() -> i32
 { unsafe { ll::savetty() } }
 
 
-pub fn scr_dump(filename: &str) -> i32
-{ unsafe { ll::scr_dump(filename.to_c_str().as_ptr()) } }
+pub fn scr_dump(filename: &str) -> Result<i32, std::ffi::NulError>
+{ unsafe { Ok(ll::scr_dump(filename.to_c_str()?.as_ptr())) } }
 
 
-pub fn scr_init(filename: &str) -> i32
-{ unsafe { ll::scr_init(filename.to_c_str().as_ptr()) } }
+pub fn scr_init(filename: &str) -> Result<i32, std::ffi::NulError>
+{ unsafe { Ok(ll::scr_init(filename.to_c_str()?.as_ptr())) } }
 
 
 pub fn scrl(n: i32) -> i32
@@ -1176,25 +1175,25 @@ pub fn scrollok(w: WINDOW, bf: bool) -> i32
 { unsafe { ll::scrollok(w, bf as ll::c_bool) } }
 
 
-pub fn scr_restore(filename: &str) -> i32
-{ unsafe { ll::scr_restore(filename.to_c_str().as_ptr()) } }
+pub fn scr_restore(filename: &str) -> Result<i32, std::ffi::NulError>
+{ unsafe { Ok(ll::scr_restore(filename.to_c_str()?.as_ptr())) } }
 
 
-pub fn scr_set(filename: &str) -> i32
-{ unsafe { ll::scr_set(filename.to_c_str().as_ptr()) } }
+pub fn scr_set(filename: &str) -> Result<i32, std::ffi::NulError>
+{ unsafe { Ok(ll::scr_set(filename.to_c_str()?.as_ptr())) } }
 
-pub fn setlocale(lc: LcCategory, locale: &str) -> String
+pub fn setlocale(lc: LcCategory, locale: &str) -> Result<String, std::ffi::NulError>
 {
   unsafe {
-    let c_str = locale.to_c_str();
+    let c_str = locale.to_c_str()?;
     let buf = c_str.as_ptr();
     let ret = libc::setlocale(lc as libc::c_int, buf);
     if ret == ptr::null_mut() {
-        String::new()
+        Ok(String::new())
     } else {
         // The clone is necessary, as the returned pointer
         // can change at any time
-        CStr::from_ptr(ret).to_string_lossy().into_owned()
+        Ok(CStr::from_ptr(ret).to_string_lossy().into_owned())
     }
   }
 }
@@ -1268,8 +1267,8 @@ pub fn slk_restore() -> i32
 { unsafe { ll::slk_restore() } }
 
 
-pub fn slk_set(n: i32, s: &str, fmt: i32) -> i32
-{ unsafe { ll::slk_set(n, s.to_c_str().as_ptr(), fmt) } }
+pub fn slk_set(n: i32, s: &str, fmt: i32) -> Result<i32, std::ffi::NulError>
+{ unsafe { Ok(ll::slk_set(n, s.to_c_str()?.as_ptr(), fmt)) } }
 
 
 pub fn slk_touch() -> i32
@@ -1324,20 +1323,20 @@ pub fn typeahead(fd: i32) -> i32
 { unsafe { ll::typeahead(fd) } }
 
 
-pub fn tigetflag(capname: &str) -> i32
-{ unsafe { ll::tigetflag(capname.to_c_str().as_ptr()) } }
+pub fn tigetflag(capname: &str) -> Result<i32, std::ffi::NulError>
+{ unsafe { Ok(ll::tigetflag(capname.to_c_str()?.as_ptr())) } }
 
 
-pub fn tigetnum(capname: &str) -> i32
-{ unsafe { ll::tigetnum(capname.to_c_str().as_ptr()) } }
+pub fn tigetnum(capname: &str) -> Result<i32, std::ffi::NulError>
+{ unsafe { Ok(ll::tigetnum(capname.to_c_str()?.as_ptr())) } }
 
 
-pub fn tigetstr(capname: &str) -> String
-{ unsafe { FromCStr::from_c_str(ll::tigetstr(capname.to_c_str().as_ptr())) } }
+pub fn tigetstr(capname: &str) -> Result<String, std::ffi::NulError>
+{ unsafe { Ok(FromCStr::from_c_str(ll::tigetstr(capname.to_c_str()?.as_ptr()))) } }
 
 
-pub fn tparm(s: &str) -> String
-{ unsafe { FromCStr::from_c_str(ll::tparm(s.to_c_str().as_ptr())) } }
+pub fn tparm(s: &str) -> Result<String, std::ffi::NulError>
+{ unsafe { Ok(FromCStr::from_c_str(ll::tparm(s.to_c_str()?.as_ptr()))) } }
 
 
 pub fn ungetch(ch: i32) -> i32
@@ -1376,12 +1375,12 @@ pub fn waddchstr(w: WINDOW, s: &[chtype]) -> i32
 { unsafe { ll::waddchstr(w, s.as_ptr()) } }
 
 
-pub fn waddnstr(w: WINDOW, s: &str, n: i32) -> i32
-{ unsafe { ll::waddnstr(w, s.to_c_str().as_ptr(), n) } }
+pub fn waddnstr(w: WINDOW, s: &str, n: i32) -> Result<i32, std::ffi::NulError>
+{ unsafe { Ok(ll::waddnstr(w, s.to_c_str()?.as_ptr(), n)) } }
 
 
-pub fn waddstr(w: WINDOW, s: &str) -> i32
-{ unsafe { ll::waddstr(w, s.to_c_str().as_ptr()) } }
+pub fn waddstr(w: WINDOW, s: &str) -> Result<i32, std::ffi::NulError>
+{ unsafe { Ok(ll::waddstr(w, s.to_c_str()?.as_ptr())) } }
 
 
 pub fn wattron(w: WINDOW, attr: NCURSES_ATTR_T) -> i32
@@ -1491,14 +1490,22 @@ pub fn wgetnstr(w: WINDOW, s: &mut String, n: i32) -> i32
 
 pub fn wgetstr(w: WINDOW, s: &mut String) -> i32
 {
-  /* XXX: This is probably broken. */
-  let mut ch = wgetch(w);
-  while ch != '\n' as i32 && ch != '\r' as i32
-  {
-    unsafe { s.as_mut_vec().push(ch as u8); }
-    ch = wgetch(w);
-  }
-  OK
+    let mut ch = wgetch(w);
+    let mut v = Vec::new();
+    while ch != '\n' as i32 && ch != '\r' as i32
+    {
+	v.push(ch as u8);
+	ch = wgetch(w);
+    }
+    match std::str::from_utf8(&v) {
+	Ok(parsed) => {
+	    *s = parsed.to_string();
+	    OK
+	},
+	Err(_) => {
+	    ERR
+	},
+    }
 }
 
 
@@ -1632,8 +1639,13 @@ pub fn wnoutrefresh(w: WINDOW) -> i32
 { unsafe { ll::wnoutrefresh(w) } }
 
 
-pub fn wprintw(w: WINDOW, s: &str) -> i32
-{ unsafe { ll::wprintw(w, s.to_c_str().as_ptr()) } }
+pub fn wprintw(w: WINDOW, s: &str) -> Result<i32, std::ffi::NulError> {
+    // We don't actually need this function to support format strings,
+    // as they are more safely done in Rust.
+    unsafe {
+	Ok(ll::wprintw(w, "%s".to_c_str()?.as_ptr(), s.to_c_str()?.as_ptr()))
+    }
+}
 
 
 pub fn wredrawln(w: WINDOW, start: i32, n: i32) -> i32
@@ -1691,73 +1703,12 @@ pub fn wgetparent(w: WINDOW) -> WINDOW
 pub fn wgetscrreg(w: WINDOW, top: &mut i32, bot: &mut i32) -> i32
 { unsafe { ll::wgetscrreg(w, &mut*top as *mut i32, &mut*bot as *mut i32) } }
 
-/* Attributes */
-pub const fn NCURSES_BITS(mask: u32, shift: u32) -> u32
-{ mask << (shift + NCURSES_ATTR_SHIFT) as usize }
 
-pub const fn A_NORMAL() -> attr_t
-{ (1u32 - 1u32) as attr_t }
-
-pub const fn A_ATTRIBUTES() -> attr_t
-{ NCURSES_BITS(!(1u32 - 1u32), 0u32) as attr_t }
-
-pub const fn A_CHARTEXT() -> attr_t
-{(NCURSES_BITS(1u32, 0u32) - 1u32) as attr_t }
-
-pub const fn A_COLOR() -> attr_t
-{ NCURSES_BITS(((1u32) << 8) - 1u32, 0u32) as attr_t }
-
-pub const fn A_STANDOUT() -> attr_t
-{ NCURSES_BITS(1u32, 8u32) as attr_t }
-
-pub const fn A_UNDERLINE() -> attr_t
-{ NCURSES_BITS(1u32, 9u32) as attr_t }
-
-pub const fn A_REVERSE() -> attr_t
-{ NCURSES_BITS(1u32, 10u32) as attr_t }
-
-pub const fn A_BLINK() -> attr_t
-{ NCURSES_BITS(1u32, 11u32) as attr_t }
-
-pub const fn A_DIM() -> attr_t
-{ NCURSES_BITS(1u32, 12u32) as attr_t }
-
-pub const fn A_BOLD() -> attr_t
-{ NCURSES_BITS(1u32, 13u32) as attr_t }
-
-pub const fn A_ALTCHARSET() -> attr_t
-{ NCURSES_BITS(1u32, 14u32) as attr_t }
-
-pub const fn A_INVIS() -> attr_t
-{ NCURSES_BITS(1u32, 15u32) as attr_t }
-
-pub const fn A_PROTECT() -> attr_t
-{ NCURSES_BITS(1u32, 16u32) as attr_t }
-
-pub const fn A_HORIZONTAL() -> attr_t
-{ NCURSES_BITS(1u32, 17u32) as attr_t }
-
-pub const fn A_LEFT() -> attr_t
-{ NCURSES_BITS(1u32, 18u32) as attr_t }
-
-pub const fn A_LOW() -> attr_t
-{ NCURSES_BITS(1u32, 19u32) as attr_t }
-
-pub const fn A_RIGHT() -> attr_t
-{ NCURSES_BITS(1u32, 20u32) as attr_t }
-
-pub const fn A_TOP() -> attr_t
-{ NCURSES_BITS(1u32, 21u32) as attr_t }
-
-pub const fn A_VERTICAL() -> attr_t
-{ NCURSES_BITS(1u32, 22u32) as attr_t }
-
-pub const fn A_ITALIC() -> attr_t
-{ NCURSES_BITS(1u32, 23u32) as attr_t }
-
-/* Colors. */
-pub fn COLOR_PAIR(n: i16) -> attr_t
-{ NCURSES_BITS(n as u32, 0u32) as attr_t }
+pub fn COLOR_PAIR(n: i16) -> attr_t {
+    unsafe {
+        ll::COLOR_PAIR(n as i32) as attr_t
+    }
+}
 
 /*
  * Most of the pseudo functions are macros that either provide compatibility
@@ -1818,154 +1769,8 @@ pub fn setsyx(y: &mut i32, x: &mut i32)
   }
 }
 
-/* Line graphics */
-pub fn NCURSES_ACS(c: char) -> chtype {
-    unsafe { *acs_map().offset((c as libc::c_uchar) as isize) as chtype }
-}
-
-/* VT100 symbols begin here */
-pub fn ACS_ULCORNER() -> chtype
-{ NCURSES_ACS('l') } /* upper left corner */
-
-pub fn ACS_LLCORNER() -> chtype
-{ NCURSES_ACS('m') } /* lower left corner */
-
-pub fn ACS_URCORNER() -> chtype
-{ NCURSES_ACS('k') } /* upper right corner */
-
-pub fn ACS_LRCORNER() -> chtype
-{ NCURSES_ACS('j') } /* lower right corner */
-
-pub fn ACS_LTEE() -> chtype
-{ NCURSES_ACS('t') } /* tee pointing right */
-
-pub fn ACS_RTEE() -> chtype
-{ NCURSES_ACS('u') } /* tee pointing left */
-
-pub fn ACS_BTEE() -> chtype
-{ NCURSES_ACS('v') } /* tee pointing up */
-
-pub fn ACS_TTEE() -> chtype
-{ NCURSES_ACS('w') } /* tee pointing down */
-
-pub fn ACS_HLINE() -> chtype
-{ NCURSES_ACS('q') } /* horizontal line */
-
-pub fn ACS_VLINE() -> chtype
-{ NCURSES_ACS('x') } /* vertical line */
-
-pub fn ACS_PLUS() -> chtype
-{ NCURSES_ACS('n') } /* large plus or crossover */
-
-pub fn ACS_S1() -> chtype
-{ NCURSES_ACS('o') } /* scan line 1 */
-
-pub fn ACS_S9() -> chtype
-{ NCURSES_ACS('s') } /* scan line 9 */
-
-pub fn ACS_DIAMOND() -> chtype
-{ NCURSES_ACS('`') } /* diamond */
-
-pub fn ACS_CKBOARD() -> chtype
-{ NCURSES_ACS('a') } /* checker board(stipple) */
-
-pub fn ACS_DEGREE() -> chtype
-{ NCURSES_ACS('f') } /* degree symbol */
-
-pub fn ACS_PLMINUS() -> chtype
-{ NCURSES_ACS('g') } /* plus/minus */
-
-pub fn ACS_BULLET() -> chtype
-{ NCURSES_ACS('~') } /* bullet */
-
-/* Teletype 5410v1 symbols begin here */
-pub fn ACS_LARROW() -> chtype
-{ NCURSES_ACS(',') } /* arrow pointing left */
-
-pub fn ACS_RARROW() -> chtype
-{ NCURSES_ACS('+') } /* arrow pointing right */
-
-pub fn ACS_DARROW() -> chtype
-{ NCURSES_ACS('.') } /* arrow pointing down */
-
-pub fn ACS_UARROW() -> chtype
-{ NCURSES_ACS('-') } /* arrow pointing up */
-
-pub fn ACS_BOARD() -> chtype
-{ NCURSES_ACS('h') } /* board of squares */
-
-pub fn ACS_LANTERN() -> chtype
-{ NCURSES_ACS('i') } /* lantern symbol */
-
-pub fn ACS_BLOCK() -> chtype
-{ NCURSES_ACS('0') } /* solid square block */
-
-/*
- * These aren't documented, but a lot of System Vs have them anyway
- * (you can spot pprryyzz{{||}} in a lot of AT&T terminfo strings).
- * The ACS_names may not match AT&T's, our source didn't know them.
- */
-pub fn ACS_S3() -> chtype
-{ NCURSES_ACS('p') } /* scan line 3 */
-
-pub fn ACS_S7() -> chtype
-{ NCURSES_ACS('r') } /* scan line 7 */
-
-pub fn ACS_LEQUAL() -> chtype
-{ NCURSES_ACS('y') } /* less/equal */
-
-pub fn ACS_GEQUAL() -> chtype
-{ NCURSES_ACS('z') } /* greater/equal */
-
-pub fn ACS_PI() -> chtype
-{ NCURSES_ACS('{') } /* Pi */
-
-pub fn ACS_NEQUAL() -> chtype
-{ NCURSES_ACS('|') } /* not equal */
-
-pub fn ACS_STERLING() -> chtype
-{ NCURSES_ACS('}') } /* UK pound sign */
-
-/*
- * Line drawing ACS names are of the form ACS_trbl, where t is the top, r
- * is the right, b is the bottom, and l is the left. t, r, b, and l might
- * be B(blank), S(single), D(double), or T(thick). The subset defined
- * here only uses B and S.
- */
-pub fn ACS_BSSB() -> chtype
-{ ACS_ULCORNER() }
-
-pub fn ACS_SSBB() -> chtype
-{ ACS_LLCORNER() }
-
-pub fn ACS_BBSS() -> chtype
-{ ACS_URCORNER() }
-
-pub fn ACS_SBBS() -> chtype
-{ ACS_LRCORNER() }
-
-pub fn ACS_SBSS() -> chtype
-{ ACS_RTEE() }
-
-pub fn ACS_SSSB() -> chtype
-{ ACS_LTEE() }
-
-pub fn ACS_SSBS() -> chtype
-{ ACS_BTEE() }
-
-pub fn ACS_BSSS() -> chtype
-{ ACS_TTEE() }
-
-pub fn ACS_BSBS() -> chtype
-{ ACS_HLINE() }
-
-pub fn ACS_SBSB() -> chtype
-{ ACS_VLINE() }
-
-pub fn ACS_SSSS() -> chtype
-{ ACS_PLUS() }
-
-pub fn KEY_F(n: u8) -> i32
+#[inline]
+pub const fn KEY_F(n: u8) -> i32
 {
   assert!(n < 16);
   KEY_F0 + n as i32
@@ -1989,8 +1794,10 @@ pub fn mouseinterval(n: i32) -> i32
 
 pub fn mousemask(newmask: mmask_t, oldmask: Option<&mut mmask_t>) -> mmask_t
 {
-    if oldmask.is_none() { unsafe { ll::mousemask(newmask, ptr::null_mut()) } }
-    else { unsafe { ll::mousemask(newmask, oldmask.unwrap()) } }
+    match oldmask {
+	None => { unsafe { ll::mousemask(newmask, ptr::null_mut()) } },
+	Some(old) => { unsafe { ll::mousemask(newmask, old) } },
+    }
 }
 
 pub fn wenclose(w: WINDOW, y: i32, x: i32) -> bool
@@ -2021,3 +1828,134 @@ pub fn extended_color_content(color: i32, r: &mut i32, g: &mut i32, b: &mut i32)
 pub fn extended_pair_content(pair: i32, f: &mut i32, b: &mut i32) -> i32 {
     unsafe { ll::extended_pair_content(pair, f, b) }
 }
+
+
+pub fn ACS_ULCORNER() -> crate::ll::chtype {
+    unsafe { crate::ll::impl_ACS_ULCORNER() }
+}
+pub fn ACS_LLCORNER() -> crate::ll::chtype {
+    unsafe { crate::ll::impl_ACS_LLCORNER() }
+}
+pub fn ACS_URCORNER() -> crate::ll::chtype {
+    unsafe { crate::ll::impl_ACS_URCORNER() }
+}
+pub fn ACS_LRCORNER() -> crate::ll::chtype {
+    unsafe { crate::ll::impl_ACS_LRCORNER() }
+}
+pub fn ACS_LTEE() -> crate::ll::chtype {
+    unsafe { crate::ll::impl_ACS_LTEE() }
+}
+pub fn ACS_RTEE() -> crate::ll::chtype {
+    unsafe { crate::ll::impl_ACS_RTEE() }
+}
+pub fn ACS_BTEE() -> crate::ll::chtype {
+    unsafe { crate::ll::impl_ACS_BTEE() }
+}
+pub fn ACS_TTEE() -> crate::ll::chtype {
+    unsafe { crate::ll::impl_ACS_TTEE() }
+}
+pub fn ACS_HLINE() -> crate::ll::chtype {
+    unsafe { crate::ll::impl_ACS_HLINE() }
+}
+pub fn ACS_VLINE() -> crate::ll::chtype {
+    unsafe { crate::ll::impl_ACS_VLINE() }
+}
+pub fn ACS_PLUS() -> crate::ll::chtype {
+    unsafe { crate::ll::impl_ACS_PLUS() }
+}
+pub fn ACS_S1() -> crate::ll::chtype {
+    unsafe { crate::ll::impl_ACS_S1() }
+}
+pub fn ACS_S9() -> crate::ll::chtype {
+    unsafe { crate::ll::impl_ACS_S9() }
+}
+pub fn ACS_DIAMOND() -> crate::ll::chtype {
+    unsafe { crate::ll::impl_ACS_DIAMOND() }
+}
+pub fn ACS_CKBOARD() -> crate::ll::chtype {
+    unsafe { crate::ll::impl_ACS_CKBOARD() }
+}
+pub fn ACS_DEGREE() -> crate::ll::chtype {
+    unsafe { crate::ll::impl_ACS_DEGREE() }
+}
+pub fn ACS_PLMINUS() -> crate::ll::chtype {
+    unsafe { crate::ll::impl_ACS_PLMINUS() }
+}
+pub fn ACS_BULLET() -> crate::ll::chtype {
+    unsafe { crate::ll::impl_ACS_BULLET() }
+}
+pub fn ACS_LARROW() -> crate::ll::chtype {
+    unsafe { crate::ll::impl_ACS_LARROW() }
+}
+pub fn ACS_RARROW() -> crate::ll::chtype {
+    unsafe { crate::ll::impl_ACS_RARROW() }
+}
+pub fn ACS_DARROW() -> crate::ll::chtype {
+    unsafe { crate::ll::impl_ACS_DARROW() }
+}
+pub fn ACS_UARROW() -> crate::ll::chtype {
+    unsafe { crate::ll::impl_ACS_UARROW() }
+}
+pub fn ACS_BOARD() -> crate::ll::chtype {
+    unsafe { crate::ll::impl_ACS_BOARD() }
+}
+pub fn ACS_LANTERN() -> crate::ll::chtype {
+    unsafe { crate::ll::impl_ACS_LANTERN() }
+}
+pub fn ACS_BLOCK() -> crate::ll::chtype {
+    unsafe { crate::ll::impl_ACS_BLOCK() }
+}
+pub fn ACS_S3() -> crate::ll::chtype {
+    unsafe { crate::ll::impl_ACS_S3() }
+}
+pub fn ACS_S7() -> crate::ll::chtype {
+    unsafe { crate::ll::impl_ACS_S7() }
+}
+pub fn ACS_LEQUAL() -> crate::ll::chtype {
+    unsafe { crate::ll::impl_ACS_LEQUAL() }
+}
+pub fn ACS_GEQUAL() -> crate::ll::chtype {
+    unsafe { crate::ll::impl_ACS_GEQUAL() }
+}
+pub fn ACS_PI() -> crate::ll::chtype {
+    unsafe { crate::ll::impl_ACS_PI() }
+}
+pub fn ACS_NEQUAL() -> crate::ll::chtype {
+    unsafe { crate::ll::impl_ACS_NEQUAL() }
+}
+pub fn ACS_STERLING() -> crate::ll::chtype {
+    unsafe { crate::ll::impl_ACS_STERLING() }
+}
+pub fn ACS_BSSB() -> crate::ll::chtype {
+    unsafe { crate::ll::impl_ACS_BSSB() }
+}
+pub fn ACS_SSBB() -> crate::ll::chtype {
+    unsafe { crate::ll::impl_ACS_SSBB() }
+}
+pub fn ACS_BBSS() -> crate::ll::chtype {
+    unsafe { crate::ll::impl_ACS_BBSS() }
+}
+pub fn ACS_SBBS() -> crate::ll::chtype {
+    unsafe { crate::ll::impl_ACS_SBBS() }
+}
+pub fn ACS_SBSS() -> crate::ll::chtype {
+    unsafe { crate::ll::impl_ACS_SBSS() }
+}
+pub fn ACS_SSSB() -> crate::ll::chtype {
+    unsafe { crate::ll::impl_ACS_SSSB() }
+}
+pub fn ACS_SSBS() -> crate::ll::chtype {
+    unsafe { crate::ll::impl_ACS_SSBS() }
+}
+pub fn ACS_BSSS() -> crate::ll::chtype {
+    unsafe { crate::ll::impl_ACS_BSSS() }
+}
+pub fn ACS_BSBS() -> crate::ll::chtype {
+    unsafe { crate::ll::impl_ACS_BSBS() }
+}
+pub fn ACS_SBSB() -> crate::ll::chtype {
+    unsafe { crate::ll::impl_ACS_SBSB() }
+}
+pub fn ACS_SSSS() -> crate::ll::chtype {
+    unsafe { crate::ll::impl_ACS_SSSS() }
+}
