@@ -5,9 +5,10 @@
 //! [apple-doc]: https://developer.apple.com/documentation/iosurface/
 //! [framework-crates]: https://docs.rs/objc2/latest/objc2/topics/about_generated/index.html
 #![no_std]
-#![cfg_attr(docsrs, feature(doc_auto_cfg))]
+#![cfg_attr(feature = "unstable-darwin-objc", feature(darwin_objc))]
+#![cfg_attr(docsrs, feature(doc_cfg))]
 // Update in Cargo.toml as well.
-#![doc(html_root_url = "https://docs.rs/objc2-io-surface/0.3.0")]
+#![doc(html_root_url = "https://docs.rs/objc2-io-surface/0.3.2")]
 
 #[cfg(feature = "alloc")]
 extern crate alloc;
