@@ -11,10 +11,10 @@ use crate::{
     ClientError, ClientResult, OAuth, Token,
 };
 
-use std::collections::HashMap;
 use std::{
+    collections::HashMap,
     io::{BufRead, BufReader, Write},
-    net::{IpAddr, SocketAddr, TcpListener},
+    net::{IpAddr, Ipv4Addr, Ipv6Addr, SocketAddr, TcpListener},
 };
 
 use maybe_async::maybe_async;
@@ -152,33 +152,33 @@ pub trait OAuthClient: BaseClient {
     // If the specified `redirect_url` is HTTP, loopback, and contains a port,
     // then the corresponding socket address is returned.
     fn get_socket_address(&self, redirect_url: &str) -> Option<SocketAddr> {
-        let url = match Url::parse(redirect_url) {
-            Ok(parsed_url) if parsed_url.scheme() == "http" && parsed_url.port().is_some() => {
-                parsed_url
-            }
-            _ => return None,
+        let (host, port) = {
+            let parsed_url = Url::parse(redirect_url).ok()?;
+            let port = match parsed_url.scheme() {
+                "http" => parsed_url.port().unwrap_or(80),
+                "https" => parsed_url.port().unwrap_or(443),
+                _ => return None,
+            };
+            (String::from(parsed_url.host_str()?), port)
         };
 
-        let host = url.host_str()?;
-        let port = url.port()?;
-
         // Handle IPv6 addresses (they come with brackets from host_str())
-        let ip_addr = if host.starts_with('[') && host.ends_with(']') {
+        let (ip_addr, fallback_ip) = if host.starts_with('[') && host.ends_with(']') {
             // Remove the brackets for IPv6 address parsing
             let ip_str = &host[1..host.len() - 1];
-            ip_str.parse::<IpAddr>().ok()?
+            (ip_str.parse::<IpAddr>(), IpAddr::V6(Ipv6Addr::UNSPECIFIED))
         } else {
             // Regular IPv4 address
-            host.parse::<IpAddr>().ok()?
+            (host.parse::<IpAddr>(), IpAddr::V4(Ipv4Addr::UNSPECIFIED))
         };
 
-        let socket_addr = SocketAddr::new(ip_addr, port);
+        let socket_addr = SocketAddr::new(ip_addr.unwrap_or(fallback_ip), port);
 
         // Return the address only if it's a loopback address
         if socket_addr.ip().is_loopback() {
             Some(socket_addr)
         } else {
-            None
+            Some(SocketAddr::new(fallback_ip, port))
         }
     }
 
@@ -1432,37 +1432,50 @@ mod tests {
 
     use super::*;
 
-    #[test]
-    #[wasm_bindgen_test]
-    fn get_socket_address_none() {
-        // A random state is generated by default
-        let spotify = AuthCodeSpotify::default();
-        // No port
-        assert_eq!(spotify.get_socket_address("http://127.0.0.1/foo"), None);
-        assert_eq!(spotify.get_socket_address("http://127.0.0.1:/foo"), None);
-        assert_eq!(spotify.get_socket_address("http://[::1]/foo"), None);
-        // Not localhost
-        assert_eq!(spotify.get_socket_address("http://56.0.0.1:1234/foo"), None);
-        assert_eq!(
-            spotify.get_socket_address("http://[3ffe:2a00:100:7031::1]:1234/foo"),
-            None
-        );
-        // Not http
-        assert_eq!(spotify.get_socket_address("https://127.0.0.1/foo"), None);
-    }
-
     #[test]
     #[wasm_bindgen_test]
     fn get_socket_address_localhost() {
         // A random state is generated by default
         let spotify = AuthCodeSpotify::default();
-        let localhost_v4 = SocketAddr::new(IpAddr::V4(Ipv4Addr::new(127, 0, 0, 1)), 1234);
-        let localhost_v6 = SocketAddr::new(IpAddr::V6(Ipv6Addr::new(0, 0, 0, 0, 0, 0, 0, 1)), 8888);
+        let localhost_v4 = SocketAddr::new(IpAddr::V4(Ipv4Addr::LOCALHOST), 1234);
+        let every_v4 = SocketAddr::new(IpAddr::V4(Ipv4Addr::UNSPECIFIED), 1234);
+        let localhost_v4_default_http_port = SocketAddr::new(IpAddr::V4(Ipv4Addr::LOCALHOST), 80);
+        let localhost_v4_default_https_port = SocketAddr::new(IpAddr::V4(Ipv4Addr::LOCALHOST), 443);
+        let every_v4_default_https_port = SocketAddr::new(IpAddr::V4(Ipv4Addr::UNSPECIFIED), 443);
+        let localhost_v6 = SocketAddr::new(IpAddr::V6(Ipv6Addr::LOCALHOST), 8888);
+        let every_v6 = SocketAddr::new(IpAddr::V6(Ipv6Addr::UNSPECIFIED), 8888);
+        let localhost_v6_default_port = SocketAddr::new(IpAddr::V6(Ipv6Addr::LOCALHOST), 80);
 
+        assert_eq!(
+            spotify.get_socket_address("http://127.0.0.1/foo"),
+            Some(localhost_v4_default_http_port)
+        );
+
+        assert_eq!(
+            spotify.get_socket_address("http://127.0.0.1:/foo"),
+            Some(localhost_v4_default_http_port)
+        );
+        assert_eq!(
+            spotify.get_socket_address("https://127.0.0.1/foo"),
+            Some(localhost_v4_default_https_port)
+        );
+        assert_eq!(
+            spotify.get_socket_address("https://google.it/foo"),
+            Some(every_v4_default_https_port)
+        );
+        assert_eq!(
+            spotify.get_socket_address("http://56.0.0.1:1234/foo"),
+            Some(every_v4)
+        );
         assert_eq!(
             spotify.get_socket_address("http://127.0.0.1:1234/foo"),
             Some(localhost_v4)
         );
+
+        assert_eq!(
+            spotify.get_socket_address("http://[::1]/foo"),
+            Some(localhost_v6_default_port)
+        );
         assert_eq!(
             spotify.get_socket_address("http://[0:0:0:0:0:0:0:1]:8888/foo"),
             Some(localhost_v6)
@@ -1471,5 +1484,9 @@ mod tests {
             spotify.get_socket_address("http://[::1]:8888/foo"),
             Some(localhost_v6)
         );
+        assert_eq!(
+            spotify.get_socket_address("http://[3ffe:2a00:100:7031::1]:8888/foo"),
+            Some(every_v6)
+        );
     }
 }
